# Copyright 2026 Flyto2. Licensed under Apache-2.0. See LICENSE.

"""
Core Training Package - Stub for OSS version

Full implementation available in Flyto Pro.
"""

from .daily_practice import DailyPracticeEngine

__all__ = ["DailyPracticeEngine"]
